import {create, select} from "d3-selection";

let defs, x_diagonal, y_diagonal, x_grid, y_grid, x_dots, y_dots;

function initPatterns() {
// these patterns were got from the textures npm package https://www.npmjs.com/package/textures
	x_diagonal = create("svg:pattern")
		.attr("id", "fl-pattern-diagonal-x")
		.attr("width", 4)
		.attr("height", 4)
		.attr("patternUnits", "userSpaceOnUse");

	x_diagonal
		.append("path")
		.attr("d", "M 0,4 l 4,-4 M -1,1 l 2,-2 M 3,5 l 2,-2")
		.attr("stroke-width", 1)
		.attr("shape-rendering", "auto")
		.attr("stroke-linecap", "square");

	x_grid = create("svg:pattern")
		.attr("id", "fl-pattern-grid-x")
		.attr("width", 4)
		.attr("height", 4)
		.attr("patternUnits", "userSpaceOnUse");

	x_grid
		.append("path")
		.attr("d", "M 2, 0 l 0, 4")
		.attr("stroke-width", 1)
		.attr("shape-rendering", "auto")
		.attr("stroke-linecap", "square");

	x_grid
		.append("path")
		.attr("d", "M 0,2 l 4,0")
		.attr("stroke-width", 1)
		.attr("shape-rendering", "auto")
		.attr("stroke-linecap", "square");

	x_dots = create("svg:pattern")
		.attr("id", "fl-pattern-dots-x")
		.attr("width", 5)
		.attr("height", 5)
		.attr("patternUnits", "userSpaceOnUse");

	x_dots
		.append("circle")
		.attr("cx", 2.5)
		.attr("cy", 2.5)
		.attr("r", 1)
		.attr("stroke-width", 1)
		.attr("shape-rendering", "auto");

	y_diagonal = select(x_diagonal.node().cloneNode(true)).attr("id", "fl-pattern-diagonal-y");
	y_grid = select(x_grid.node().cloneNode(true)).attr("id", "fl-pattern-grid-y");
	y_dots= select(x_dots.node().cloneNode(true)).attr("id", "fl-pattern-dots-y");

	const patterns = [x_diagonal, x_dots, x_grid, y_diagonal, y_grid, y_dots];

	defs = create("svg:defs").node();

	patterns.forEach(p => {
		defs.appendChild(p.node());
	});
}

export function initAppendPatternsTo(instance) {
	let add_patterns_to = instance.appendTo();
	initPatterns();

	instance.appendPatternsTo = function(container) {
		if (container === undefined) return add_patterns_to;
		container.node().appendChild(defs);
		add_patterns_to = container;
		return instance;
	};
}


export function updatePatterns(state) {
	x_dots.select("circle").style("fill", state.x_area_color);
	x_diagonal.select("path").style("stroke", state.x_area_color);
	x_grid.selectAll("path").style("stroke", state.x_area_color);
	y_dots.select("circle").style("fill", state.y_area_color);
	y_diagonal.select("path").style("stroke", state.y_area_color);
	y_grid.selectAll("path").style("stroke", state.y_area_color);
}
